module vga_driver(
					clk,
					rst_n,
					hys,
					vys,
				    lcd_rgb 
               );
parameter     PICTURE_W    =   16 ;
parameter     ROW_W        =   10;

parameter     TIME_1S      = 25000000;


input                  clk      ;
input                  rst_n    ;

output                 hys      ;
output                 vys      ;


output [PICTURE_W-1:0] lcd_rgb;
reg    [PICTURE_W-1:0] lcd_rgb;

reg    [ROW_W-1:0]     h_cnt;
reg    [ROW_W-1:0]     v_cnt;
reg                    hys;
reg                    vys;
reg                    valid_area ;

reg[24:0]              cnt_1s;
reg[ 4:0]              cnt_12s; 
reg[16:0]              lcd_data_1st;
reg[16:0]              lcd_data_2nd;
reg[16:0]              lcd_data_3rd;
reg[16:0]              lcd_data_4th;




always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        cnt_1s <= 0;
    end
    else if(cnt_1s==TIME_1S-1) begin
        cnt_1s <= 0;
    end
    else begin
        cnt_1s <= cnt_1s + 1;
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        cnt_12s <= 0;
    end
    else if(cnt_1s==TIME_1S-1) begin
        if(cnt_12s==15)
            cnt_12s <= 0;
        else
            cnt_12s <= cnt_12s + 1;
    end
end

always@(posedge clk or negedge rst_n)begin
	if(!rst_n)begin
      	h_cnt <= 10'd0;
	 end
    else if(h_cnt == 10'd799)begin
        h_cnt <= 10'd0;
     end
    else begin
	    h_cnt <= h_cnt+1'b1;
     end
 end

 always@(posedge clk or negedge rst_n)begin
	if(!rst_n) begin
       v_cnt <= 10'd0;
    end
    else if(h_cnt == 10'd799)begin
        if(v_cnt==524)
            v_cnt <= 0;
        else
            v_cnt <= v_cnt+1;
    end
 end
	

always@(posedge clk or negedge rst_n)begin
    if(!rst_n)begin
        hys <= 0;
    end
    else if(h_cnt==10'd95)begin
        hys <= 1'b1;
    end
    else if(h_cnt==10'd799)begin
        hys <= 1'b0;
    end
end
	


always@(posedge clk or negedge rst_n)begin
    if(!rst_n)begin
        vys <= 1'b0;
    end
    else if(v_cnt<10'd2)begin
        vys <= 1'b0;
    end
    else begin
        vys <= 1'b1;
    end
end    

always  @(*)begin
    valid_area = h_cnt >=141 && h_cnt <=786 && v_cnt >=32 && v_cnt <=515;
end


always  @(*)begin
    lcd_data_1st  = {PICTURE_W{1'b1}};
end

always  @(*)begin
    lcd_data_2nd = 16'b00000_111111_00000;
end

always  @(*)begin
    if((v_cnt>=32 && v_cnt<274) && (h_cnt>=141 && h_cnt<786))begin
        lcd_data_3rd = 16'b11111_000000_00000;
    end
    else if((v_cnt>=274 && v_cnt<515) && (h_cnt>=141 && h_cnt<786))begin
        lcd_data_3rd = 16'b00000_000000_11111;
    end
end

always  @(*)begin
    if((v_cnt>=32 && v_cnt<274) && (h_cnt>=141 && h_cnt<463))begin
        lcd_data_4th = 16'b11111_000000_00000;
    end
    else if((v_cnt>=32 && v_cnt<274) && (h_cnt>=464 && h_cnt<=786))begin
        lcd_data_4th <= 16'b00000_000000_11111;
    end
    else if((v_cnt>=274 && v_cnt<=515) && (h_cnt>=141 && h_cnt<463))begin
        lcd_data_4th <= 16'b00000_111111_00000;
    end
    else if((v_cnt>=274 && v_cnt<=515) && (h_cnt>=463 && h_cnt<=786))begin
        lcd_data_4th <= 16'b11111_111111_00000;
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        lcd_rgb <= 8'h0;
    end
    else if(valid_area)begin
        if(cnt_12s <4)
            lcd_rgb <= lcd_data_1st;
        else if(cnt_12s <8)
            lcd_rgb <= lcd_data_2nd;
        else if(cnt_12s <12)
            lcd_rgb <= lcd_data_3rd;
        else
            lcd_rgb <= lcd_data_4th;
    end
    else begin
        lcd_rgb <= 0;
    end
end


endmodule
